#!/usr/bin/env bash

# Import functions and environment variables
LIMINE_FUNCTIONS_PATH=/usr/lib/limine/limine-common-functions
# shellcheck disable=SC1090
source "${LIMINE_FUNCTIONS_PATH}" || {
	echo -e "\033[1;31m Error: Failed to source '${LIMINE_FUNCTIONS_PATH}'.\033[0m" >&2
	exit 1
}

initialize_header || exit 1

all=0
lines=()

# Read input and identify whether to process all modules or specific changes
while read -r line; do
	if [[ "${line}" != */vmlinuz ]]; then
		# Trigger for changes affecting mkinitcpio files
		all=1
		continue
	fi
	lines+=("/${line%/vmlinuz}")
done

if ((all)); then
	lines=(/usr/lib/modules/*)
fi

mutex_lock "limine-mkinitcpio-install -> reset_enroll_config"
reset_enroll_config
mutex_unlock

# Process each kernel directory
for line in "${lines[@]}"; do
	if ! pacman -Qqo "${line}/pkgbase" &>/dev/null; then
		# Skip kernel if pkgBase does not belong to any package
		continue
	fi

	pkgBase="$(<"${line}/pkgbase")"
	kVer="${line##*/}"

	ukiDirPath="${ESP_PATH}/EFI/Linux"
	kDirPath="${ESP_PATH}/${MACHINE_ID}/${pkgBase}"

	if [[ "$ENABLE_UKI" == "yes" ]]; then
		# Unified Kernel Image (UKI)
		install -dm700 "${ukiDirPath}"

		echo "Building UKI for ${pkgBase} (${kVer})"
		# Note: UKI ignores the external cmdline and does not boot into a Btrfs snapshot when Secure Boot is enabled. This is why the solution is here:
		mutex_lock "limine-mkinitcpio-install -> UKI"
		if is_snapper_and_sb_installed; then
		  mkinitcpio --kernel "${kVer}" --no-cmdline --uki "${ukiDirPath}/${MACHINE_ID}_${pkgBase}.efi"
		else
		  get_kernel_cmdline "${pkgBase}" > /tmp/kernel-cmdline
		  mkinitcpio --kernel "${kVer}" --uki "${ukiDirPath}/${MACHINE_ID}_${pkgBase}.efi" --cmdline /tmp/kernel-cmdline
		  rm /tmp/kernel-cmdline
		fi
		sb_sign "${ukiDirPath}/${MACHINE_ID}_${pkgBase}.efi"
		mutex_unlock
		limine-entry-tool --add-uki "${pkgBase}" "${ukiDirPath}/${MACHINE_ID}_${pkgBase}.efi" --comment "${kVer}"
		echo "UKI stored in: ${ukiDirPath}/${MACHINE_ID}_${pkgBase}.efi"

		# Remove existing initramfs and vmlinuz
		if [[ -d "${kDirPath}" ]]; then
			limine-entry-tool --remove "${pkgBase}"
		fi

	else

		# Initramfs and vmlinuz handling
		install -dm700 "${kDirPath}"
		install -Dm600 "${line}/vmlinuz" "${kDirPath}/vmlinuz-${pkgBase}"

		echo "Building initramfs for ${pkgBase} (${kVer})"
		mutex_lock "limine-mkinitcpio-install -> initramfs"
		mkinitcpio --kernel "${kVer}" --no-cmdline --generate "${kDirPath}/initramfs-${pkgBase}"
		mutex_unlock
		limine-entry-tool --add "${pkgBase}" "${kDirPath}/initramfs-${pkgBase}" "${kDirPath}/vmlinuz-${pkgBase}" --comment "${kVer}"
		echo "Kernel stored in: ${kDirPath}/vmlinuz-${pkgBase}"
		echo "Initramfs stored in: ${kDirPath}/initramfs-${pkgBase}"

		# Remove existing UKI files.
		if [[ -f "${ukiDirPath}/${MACHINE_ID}_${pkgBase}.efi" ]]; then
			limine-entry-tool --remove-uki "${pkgBase}"
		fi
		if [[ -f "${ukiDirPath}/${MACHINE_ID}_${pkgBase}-fallback.efi" ]]; then
			limine-entry-tool --remove-uki "${pkgBase}-fallback"
		fi
	fi
done

mutex_lock "limine-mkinitcpio-install -> enroll_config"
enroll_config
mutex_unlock